#include "bintree.h"
#include <catch.h>

using namespace bst;

TEST_CASE("Nothing is member of empty")
{
    Tree tree;
    CHECK_FALSE(member(tree, "a"));
}

TEST_CASE("Lookup of non-member throws")
{
    Tree tree;
    CHECK_THROWS_AS(lookup(tree, "a"), Not_found);
}

TEST_CASE("Inserted member is member, not other is not")
{
    Tree tree;
    insert(tree, "b", 5);

    CHECK_FALSE(member(tree, "a"));
    CHECK(member(tree, "b"));
    CHECK(5 == lookup(tree, "b"));
}

TEST_CASE("Long scenario of insertions and checks")
{
    Tree tree;
    insert(tree, "d", 4);
    insert(tree, "b", 2);
    insert(tree, "a", 1);

    CHECK(member(tree, "a"));
    CHECK(member(tree, "b"));
    CHECK_FALSE(member(tree, "c"));
    CHECK(member(tree, "d"));
    CHECK_FALSE(member(tree, "e"));
    CHECK_FALSE(member(tree, "f"));
    CHECK_FALSE(member(tree, "g"));

    insert(tree, "c", 3);
    insert(tree, "f", 6);
    insert(tree, "e", 5);
    insert(tree, "g", 7);

    CHECK(member(tree, "a"));
    CHECK(member(tree, "b"));
    CHECK(member(tree, "c"));
    CHECK(member(tree, "d"));
    CHECK(member(tree, "e"));
    CHECK(member(tree, "f"));
    CHECK(member(tree, "g"));
    CHECK_FALSE(member(tree, ""));
    CHECK_FALSE(member(tree, "ax"));
    CHECK_FALSE(member(tree, "bx"));
    CHECK_FALSE(member(tree, "cx"));
    CHECK_FALSE(member(tree, "dx"));
    CHECK_FALSE(member(tree, "ex"));
    CHECK_FALSE(member(tree, "fx"));
    CHECK_FALSE(member(tree, "gx"));

    CHECK(1 == lookup(tree, "a"));
    CHECK(2 == lookup(tree, "b"));
    CHECK(3 == lookup(tree, "c"));
    CHECK(4 == lookup(tree, "d"));
    CHECK(5 == lookup(tree, "e"));
    CHECK(6 == lookup(tree, "f"));
    CHECK(7 == lookup(tree, "g"));

    CHECK_THROWS_AS(lookup(tree, "h"), Not_found);
}

