#include "string_list.h"

namespace string_list {

bool is_empty(List lst)
{
    return lst == nullptr;
}

bool is_cons(List lst)
{
    return lst != nullptr;
}

// Recursive length is simple, but it uses auxiliary space (on the stack)
// that is linear in the size of `lst`. The iterative version, below, runs
// in constant space.
size_t length_rec(List lst)
{
    if (is_empty(lst))
        return 0;
    else
        return 1 + length_rec(lst->rest);
}

// Here is the iterative length function. It uses a loop to traverse the list.
size_t length(List lst)
{
    size_t result = 0;

    while (is_cons(lst)) {
        ++result;
        lst = lst->rest;
    }

    return result;
}

// Recursive append is simple, but it uses auxiliary space (on the stack)
// that is linear in the size of `before`. The iterative version, below, runs
// in constant space.
List append_rec(List before, List after)
{
    if (is_empty(before))
        return after;
    else
        return cons(before->first, append(before->rest, after));
}

List append(List before, List after)
{
    if (is_empty(before)) {
        return after;
    } else {

        // The idea here is to allocate each new cons cell with nullptr for
        // its rest, and then always hold onto the most recently allocated
        // cons cell so that we can update its rest to be the next one.
        // Our result will be the first cons cell, and `prev` will track the
        // most recently allocated one.
        List result = cons(before->first, nullptr);
        List prev = result;

        // Now we traverse the rest of `before` (since we already copied
        // `before->first`), consing each element in turn and keeping track
        // of the most recent cons.
        for (List curr = before->rest; is_cons(curr); curr = curr->rest) {
            prev->rest = cons(curr->first, nullptr);
            prev = prev->rest;
        }

        // At the end, hook the last cons onto `after`.
        prev->rest = after;

        return result;
    }
}

// Ditto, linear stack space.
void concat_rec(List& before, List after)
{
    if (before == nullptr)
        before = after;
    else
        concat_rec(before->rest, after);
}

void concat(List& before, List after)
{
    if (before == nullptr) {
        before = after;
    } else {
        List curr = before;
        while (curr->rest != nullptr) curr = curr->rest;
        curr->rest = after;
    }
}

List reverse(List lst)
{
    List result;

    while (is_cons(lst)) {
        result = cons(lst->first, result);
        lst = lst->rest;
    }

    return result;
}

} // end namespace string_list
